﻿namespace HeuristicLab.Problems.ProgramSynthesis.Push.Interpreter {
  using System.Collections.Generic;
  using System.Threading;
  using System.Threading.Tasks;
  using Core;
  using Expressions;

  using HeuristicLab.Problems.ProgramSynthesis.Push.Configuration;

  using Stack;

  public interface IPushInterpreter {
    IRandom Random { get; set; }
    IPushStack<Expression> CodeStack { get; }
    IPushStack<Expression> ExecStack { get; }
    IPushStack<string> NameStack { get; }
    IPushStack<bool> BooleanStack { get; }
    IPushStack<long> IntegerStack { get; }
    IPushStack<double> FloatStack { get; }
    IPushStack<char> CharStack { get; }
    IPushStack<string> StringStack { get; }
    IPushStack<List<long>> IntegerVectorStack { get; }
    IPushStack<List<double>> FloatVectorStack { get; }
    IPushStack<List<bool>> BooleanVectorStack { get; }
    IPushStack<List<string>> StringVectorStack { get; }
    IPushStack<string> PrintStack { get; }
    IDictionary<string, Expression> CustomExpressions { get; }
    IReadOnlyDictionary<StackTypes, IPushStack> Stacks { get; }
    IReadOnlyPushConfiguration Configuration { get; }
    void Clear();
    void Reset();
    void Run(string code, bool stepwise = false);
    void Run(Expression expression, bool stepwise = false);
    Task RunAsync(Expression expression, CancellationToken token = default(CancellationToken));
    Task RunAsync(string code, CancellationToken token = default(CancellationToken));
    Task AbortAndResetAsync();
    Task AbortAsync();
    Task PauseAsync();
    void Resume();
    Task ResumeAsync();
    bool Step();
  }
}